@@ -8,7 +8,7 @@
 #include "streaming.h"
 #include "internal/loaders.h"
 
-bool static claim_bytes(size_t required, size_t provided,
+static bool claim_bytes(size_t required, size_t provided,
                         struct cbor_decoder_result *result) {
   if (required > (provided - result->read)) {
     result->required = required + result->read;
@@ -22,6 +22,24 @@ bool static claim_bytes(size_t required, size_t provided,
   }
 }
 
+// Use implicit capture as an exception to avoid the super long parameter list
+#define CLAIM_BYTES_AND_INVOKE(callback_name, length, source_extra_offset) \
+  do {                                                                     \
+    if (claim_bytes(length, source_size, &result)) {                       \
+      callbacks->callback_name(context, source + 1 + source_extra_offset,  \
+                               length);                                    \
+    }                                                                      \
+  } while (0)
+
+#define READ_CLAIM_INVOKE(callback_name, length_reader, length_bytes) \
+  do {                                                                \
+    if (claim_bytes(length_bytes, source_size, &result)) {            \
+      uint64_t length = length_reader(source + 1);                    \
+      CLAIM_BYTES_AND_INVOKE(callback_name, length, length_bytes);    \
+    }                                                                 \
+    return result;                                                    \
+  } while (0)
+
 struct cbor_decoder_result cbor_stream_decode(
     cbor_data source, size_t source_size,
     const struct cbor_callbacks *callbacks, void *context) {
@@ -98,7 +116,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x1E: /* Fallthrough */
     case 0x1F:
       /* Reserved */
-      { return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR}; }
+      { return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR}; }
     case 0x20: /* Fallthrough */
     case 0x21: /* Fallthrough */
     case 0x22: /* Fallthrough */
@@ -166,7 +184,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x3E: /* Fallthrough */
     case 0x3F:
       /* Reserved */
-      { return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR}; }
+      { return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR}; }
     case 0x40: /* Fallthrough */
     case 0x41: /* Fallthrough */
     case 0x42: /* Fallthrough */
@@ -193,63 +211,27 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x57:
       /* Embedded length byte string */
       {
-        size_t length =
-            (size_t)_cbor_load_uint8(source) - 0x40; /* 0x40 offset */
-        if (claim_bytes(length, source_size, &result)) {
-          callbacks->byte_string(context, source + 1, length);
-        }
+        uint64_t length = _cbor_load_uint8(source) - 0x40; /* 0x40 offset */
+        CLAIM_BYTES_AND_INVOKE(byte_string, length, 0);
         return result;
       }
     case 0x58:
       /* One byte length byte string */
-      // TODO template this?
-      {
-        if (claim_bytes(1, source_size, &result)) {
-          size_t length = (size_t)_cbor_load_uint8(source + 1);
-          if (claim_bytes(length, source_size, &result)) {
-            callbacks->byte_string(context, source + 1 + 1, length);
-          }
-        }
-        return result;
-      }
+      READ_CLAIM_INVOKE(byte_string, _cbor_load_uint8, 1);
     case 0x59:
       /* Two bytes length byte string */
-      {
-        if (claim_bytes(2, source_size, &result)) {
-          size_t length = (size_t)_cbor_load_uint16(source + 1);
-          if (claim_bytes(length, source_size, &result)) {
-            callbacks->byte_string(context, source + 1 + 2, length);
-          }
-        }
-        return result;
-      }
+      READ_CLAIM_INVOKE(byte_string, _cbor_load_uint16, 2);
     case 0x5A:
       /* Four bytes length byte string */
-      {
-        if (claim_bytes(4, source_size, &result)) {
-          size_t length = (size_t)_cbor_load_uint32(source + 1);
-          if (claim_bytes(length, source_size, &result)) {
-            callbacks->byte_string(context, source + 1 + 4, length);
-          }
-        }
-        return result;
-      }
+      READ_CLAIM_INVOKE(byte_string, _cbor_load_uint32, 4);
     case 0x5B:
       /* Eight bytes length byte string */
-      {
-        if (claim_bytes(8, source_size, &result)) {
-          size_t length = (size_t)_cbor_load_uint64(source + 1);
-          if (claim_bytes(length, source_size, &result)) {
-            callbacks->byte_string(context, source + 1 + 8, length);
-          }
-        }
-        return result;
-      }
+      READ_CLAIM_INVOKE(byte_string, _cbor_load_uint64, 8);
     case 0x5C: /* Fallthrough */
     case 0x5D: /* Fallthrough */
     case 0x5E:
       /* Reserved */
-      { return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR}; }
+      { return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR}; }
     case 0x5F:
       /* Indefinite byte string */
       {
@@ -282,62 +264,27 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x77:
       /* Embedded one byte length string */
       {
-        size_t length =
-            (size_t)_cbor_load_uint8(source) - 0x60; /* 0x60 offset */
-        if (claim_bytes(length, source_size, &result)) {
-          callbacks->string(context, source + 1, length);
-        }
+        uint64_t length = _cbor_load_uint8(source) - 0x60; /* 0x60 offset */
+        CLAIM_BYTES_AND_INVOKE(string, length, 0);
         return result;
       }
     case 0x78:
       /* One byte length string */
-      {
-        if (claim_bytes(1, source_size, &result)) {
-          size_t length = (size_t)_cbor_load_uint8(source + 1);
-          if (claim_bytes(length, source_size, &result)) {
-            callbacks->string(context, source + 1 + 1, length);
-          }
-        }
-        return result;
-      }
+      READ_CLAIM_INVOKE(string, _cbor_load_uint8, 1);
     case 0x79:
       /* Two bytes length string */
-      {
-        if (claim_bytes(2, source_size, &result)) {
-          size_t length = (size_t)_cbor_load_uint16(source + 1);
-          if (claim_bytes(length, source_size, &result)) {
-            callbacks->string(context, source + 1 + 2, length);
-          }
-        }
-        return result;
-      }
+      READ_CLAIM_INVOKE(string, _cbor_load_uint16, 2);
     case 0x7A:
       /* Four bytes length string */
-      {
-        if (claim_bytes(4, source_size, &result)) {
-          size_t length = (size_t)_cbor_load_uint32(source + 1);
-          if (claim_bytes(length, source_size, &result)) {
-            callbacks->string(context, source + 1 + 4, length);
-          }
-        }
-        return result;
-      }
+      READ_CLAIM_INVOKE(string, _cbor_load_uint32, 4);
     case 0x7B:
       /* Eight bytes length string */
-      {
-        if (claim_bytes(8, source_size, &result)) {
-          size_t length = (size_t)_cbor_load_uint64(source + 1);
-          if (claim_bytes(length, source_size, &result)) {
-            callbacks->string(context, source + 1 + 8, length);
-          }
-        }
-        return result;
-      }
+      READ_CLAIM_INVOKE(string, _cbor_load_uint64, 8);
     case 0x7C: /* Fallthrough */
     case 0x7D: /* Fallthrough */
     case 0x7E:
       /* Reserved */
-      { return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR}; }
+      { return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR}; }
     case 0x7F:
       /* Indefinite length string */
       {
@@ -371,14 +318,14 @@ struct cbor_decoder_result cbor_stream_decode(
       /* Embedded one byte length array */
       {
         callbacks->array_start(
-            context, (size_t)_cbor_load_uint8(source) - 0x80); /* 0x40 offset */
+            context, _cbor_load_uint8(source) - 0x80); /* 0x40 offset */
         return result;
       }
     case 0x98:
       /* One byte length array */
       {
         if (claim_bytes(1, source_size, &result)) {
-          callbacks->array_start(context, (size_t)_cbor_load_uint8(source + 1));
+          callbacks->array_start(context, _cbor_load_uint8(source + 1));
         }
         return result;
       }
@@ -386,8 +333,7 @@ struct cbor_decoder_result cbor_stream_decode(
       /* Two bytes length array */
       {
         if (claim_bytes(2, source_size, &result)) {
-          callbacks->array_start(context,
-                                 (size_t)_cbor_load_uint16(source + 1));
+          callbacks->array_start(context, _cbor_load_uint16(source + 1));
         }
         return result;
       }
@@ -395,8 +341,7 @@ struct cbor_decoder_result cbor_stream_decode(
       /* Four bytes length array */
       {
         if (claim_bytes(4, source_size, &result)) {
-          callbacks->array_start(context,
-                                 (size_t)_cbor_load_uint32(source + 1));
+          callbacks->array_start(context, _cbor_load_uint32(source + 1));
         }
         return result;
       }
@@ -404,8 +349,7 @@ struct cbor_decoder_result cbor_stream_decode(
       /* Eight bytes length array */
       {
         if (claim_bytes(8, source_size, &result)) {
-          callbacks->array_start(context,
-                                 (size_t)_cbor_load_uint64(source + 1));
+          callbacks->array_start(context, _cbor_load_uint64(source + 1));
         }
         return result;
       }
@@ -413,7 +357,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0x9D: /* Fallthrough */
     case 0x9E:
       /* Reserved */
-      { return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR}; }
+      { return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR}; }
     case 0x9F:
       /* Indefinite length array */
       {
@@ -446,15 +390,15 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xB7:
       /* Embedded one byte length map */
       {
-        callbacks->map_start(
-            context, (size_t)_cbor_load_uint8(source) - 0xA0); /* 0xA0 offset */
+        callbacks->map_start(context,
+                             _cbor_load_uint8(source) - 0xA0); /* 0xA0 offset */
         return result;
       }
     case 0xB8:
       /* One byte length map */
       {
         if (claim_bytes(1, source_size, &result)) {
-          callbacks->map_start(context, (size_t)_cbor_load_uint8(source + 1));
+          callbacks->map_start(context, _cbor_load_uint8(source + 1));
         }
         return result;
       }
@@ -462,7 +406,7 @@ struct cbor_decoder_result cbor_stream_decode(
       /* Two bytes length map */
       {
         if (claim_bytes(2, source_size, &result)) {
-          callbacks->map_start(context, (size_t)_cbor_load_uint16(source + 1));
+          callbacks->map_start(context, _cbor_load_uint16(source + 1));
         }
         return result;
       }
@@ -470,7 +414,7 @@ struct cbor_decoder_result cbor_stream_decode(
       /* Four bytes length map */
       {
         if (claim_bytes(4, source_size, &result)) {
-          callbacks->map_start(context, (size_t)_cbor_load_uint32(source + 1));
+          callbacks->map_start(context, _cbor_load_uint32(source + 1));
         }
         return result;
       }
@@ -478,7 +422,7 @@ struct cbor_decoder_result cbor_stream_decode(
       /* Eight bytes length map */
       {
         if (claim_bytes(8, source_size, &result)) {
-          callbacks->map_start(context, (size_t)_cbor_load_uint64(source + 1));
+          callbacks->map_start(context, _cbor_load_uint64(source + 1));
         }
         return result;
       }
@@ -486,7 +430,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xBD: /* Fallthrough */
     case 0xBE:
       /* Reserved */
-      { return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR}; }
+      { return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR}; }
     case 0xBF:
       /* Indefinite length map */
       {
@@ -506,8 +450,8 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xC5:
       /* Big float */
       {
-        callbacks->tag(context,
-                       _cbor_load_uint8(source) - 0xC0); /* 0xC0 offset */
+        callbacks->tag(context, (uint64_t)(_cbor_load_uint8(source) -
+                                           0xC0)); /* 0xC0 offset */
         return result;
       }
     case 0xC6: /* Fallthrough */
@@ -526,14 +470,14 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xD3: /* Fallthrough */
     case 0xD4: /* Unassigned tag value */
     {
-      return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR};
+      return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR};
     }
     case 0xD5: /* Expected b64url conversion tag - fallthrough */
     case 0xD6: /* Expected b64 conversion tag - fallthrough */
     case 0xD7: /* Expected b16 conversion tag */
     {
-      callbacks->tag(context,
-                     _cbor_load_uint8(source) - 0xC0); /* 0xC0 offset */
+      callbacks->tag(context, (uint64_t)(_cbor_load_uint8(source) -
+                                         0xC0)); /* 0xC0 offset */
       return result;
     }
     case 0xD8: /* 1B tag */
@@ -569,7 +513,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xDE: /* Fallthrough */
     case 0xDF: /* Reserved */
     {
-      return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR};
+      return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR};
     }
     case 0xE0: /* Fallthrough */
     case 0xE1: /* Fallthrough */
@@ -592,7 +536,7 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xF2: /* Fallthrough */
     case 0xF3: /* Simple value - unassigned */
     {
-      return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR};
+      return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR};
     }
     case 0xF4:
       /* False */
@@ -620,7 +564,7 @@ struct cbor_decoder_result cbor_stream_decode(
       }
     case 0xF8:
       /* 1B simple value, unassigned */
-      { return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR}; }
+      { return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR}; }
     case 0xF9:
       /* 2B float */
       {
@@ -649,16 +593,13 @@ struct cbor_decoder_result cbor_stream_decode(
     case 0xFD: /* Fallthrough */
     case 0xFE:
       /* Reserved */
-      { return (struct cbor_decoder_result){0, CBOR_DECODER_ERROR}; }
+      { return (struct cbor_decoder_result){.status = CBOR_DECODER_ERROR}; }
     case 0xFF:
       /* Break */
-      {
-        callbacks->indef_break(context);
-        return result;
-      }
-    default: /* Never happens - this shuts up the compiler */
-    {
+      callbacks->indef_break(context);
+      // Never happens, the switch statement is exhaustive on the 1B range; make
+      // compiler happy
+    default:
       return result;
-    }
   }
 }
